function coef = rayleigh(xi,yi,xmark,Hrms,Htr)
% ----------------------------------------------------------------
%     function coef = rayleigh(xi,yi,xmark)
%
%  Rayleigh Probability Paper: The paper is marked with probability
%  of exceedance [0.002, 0.005, 0.01, 0.05, 0.1:0.1:0.9]  (horizontal
%  lines associated with these values are drawn).  In addition, it draws 
%  the linear curve fitting for input data points.
%
%  xi = horizontal input (physical quantity)
%  yi = vertical input (in probability of exceedance)
%  xmark = vertical grid lines to be plotted
%  coef = slope and intercept of the linear regression line.
x = xmark;
poe = [1:-0.1:0.1, 0.05,0.02,0.01,0.005,0.002,0.001,0.0005]; % prob.of exceedance
y = sqrt(-log(poe));  
n = length(y);
m = length(x);
axis([x(1) x(m) y(1) y(n)]);
dx = abs(x(m)-x(1))/25;
xh = [x(1) x(m)];
% working on horizontal line
for k=1:n,
    yh=[y(k) y(k)];
    line(xh,yh);
    text(x(1)-2*dx,y(k),num2str(poe(k)));  % prob. of exceedance
end
% set(gca,'linestyle',':');
% working on vertical line (horizontal scale)
dy=abs(y(n)-y(1))/25;
yv=[y(1),y(n)];
for k=1:m,
    xv=[x(k) x(k)];
    line(xv,yv);
    text(x(k)-dx/4,y(1)-dy,num2str(xmark(k)));
end
axis('off');
% x is wave height
% y is probability (correct numbers)
% z is scaled probability for Rayleigh paper
% plotting data points, convert probabilities for plot
%
zi = sqrt(-log(yi));
hold on
plot(xi,zi,'.')
%
% to draw regression line
%
coef=polyfit(xi,zi,1);
xa = 0.5 * min(xi);
xb = 1.5 * max(xi);
xx = [xa xi xb];
yy = polyval(coef,xx);
plot(xx,yy,'-');
%
H1=BG(Htr/Hrms,1);
H2=BG(Htr/Hrms,2);
xxx=[xmark(1):.01:xmark(m)];
lxxx=length(xxx);
%calculate plot with CWD of Battjes Groenendijk
for j=1:lxxx
  if (xxx(j)<Htr)
    yyy(j)=1-exp(-power((xxx(j)/(H1*Hrms)),2.0));  %BG 2
    else
    yyy(j)=1-exp(-power((xxx(j)/(H2*Hrms)),3.6));  %BG 3.6
    end;
end; 
%convert probabilities to scale for plot
zzz=sqrt(-log(1-yyy));
hold on

plot(xxx,zzz,'r');



