% Script for processing pressure data from the EMS wavelogger
% Script developed by H.J. Verhagen, Delft University of Technology
% February 2013
clear;
clc;
%define basic variables
Rho=1010;
g=9.81;
Avolt=262.47;     %Calibration value A of the pressure meter;
Bvolt=-3617.6;  %Calibration value B of the pressure meter;
TanAlfa = 0.001;   %bedslope
NofFiles=1;        % number of files to be processed
StartFile=1;       % first file to be processed
Bottom=0.6;        % height of sensor above bottom
for ii=StartFile:NofFiles
 if (ii==StartFile)
     [FileName,PathName] = uigetfile('*.txt','Select the wavelogger file');
     end;
 if (NofFiles==1)
  fid=fopen([PathName,FileName]);
  [pathstr, heading,ext]=fileparts(FileName);
 else
  if(ii==StartFile)
     [pathstr, heading,ext]=fileparts(FileName);
     heading=heading(1:(length(heading)-1));
     end;
  tellerstring=int2str(ii);
  FileName=([heading,tellerstring,'.txt']);
  fid=fopen([PathName,FileName]);
  end;

     % heading is caption of figures, default heading=FileName
     % if needed assign other value to heading here
     % heading = 'fill in your heading';
                       
HDRS = textscan(fid,'%s %d %s %d %s',1, 'delimiter',',');
tijd=HDRS{3};
temp=HDRS{4};
date=HDRS{5};
ccc=1;
n=0;

while ccc==1
  DATA = textscan(fid,'%s %d %d d','delimiter',',');
  ccc=strcmpi(DATA{1},'$D');
  n=n+1;
  %transformation from Volts to pressure using calibration constants Avolt
  %and Bvolt
  %Avolt has dimension Pa/V, Bvolt has dimension Pa
  if (ccc==1) q(n)=Avolt*DATA{3}+Bvolt+0.1;
      end;
end;
n=n-1;      %count number of samples    
fclose(fid);                       
P=double(q');

% additional input data for spectrum
%===================================
M = 50;        % higher values of M give more smoothing of the spectrum
               % low value for M is 20, high value for M is 100
high= 0.175;     % highest frequency for plots  default high=0.4
low = 0.115;    % lowest frequency for plots   default low =0.05
cutoff=0.08;   % parameter to eliminate noise from the tail of the spectrum 
               % when the spectral value is less than "cutoff" % of the max
               % value, then the spectral value is set to exact zero; 
               % default = 0.08
extraplot = 1; % extraplot = 0 gives no plots, 1= only summary
               % 2=include prssure, 3=include depth, 4= include pressure
method=2;      % method=1 uses Tm for calculating waveheight from pressure
               % method=2 uses rea period per wave
steepness=0.05;% max allowed wave steepness individual waves, default 0.05
Hminimum=0.10; % minimum waveheigth to be considered default .10 m
interval=.25;  % interval is the sample frequency interval of the sensor
               % do not change this value unless another wavelogger is used
               % .25
iBegin=50;     % Start value of array of spectrum bins default 50
iEnd=2000;     % End value of array of spectrum bins default 2000              
%==========================================================================
if(NofFiles>1)extraplot=0;end; %in case of multiple files plotting individual 
                                %files is blocked
tottime=n*interval;   %calculate total duration of observation in sec
ttime=(interval:interval:tottime); %create array with time
time=ttime.';          %change orientation of matrix
% calculate regression coefficients to compensate for change in waterlevel
% during the observations

if (extraplot>3)
    figure;
    plot(time,P);
    xlabel('time (sec)');
    ylabel('pressure (Pa)');
    title(['observed total pressure ',heading]);
end;
BB=polyfit(time,P,1); %regression analysis to determine real waterdepth at any moment
                      %and correct for hydrostatic pressure 
Intercept=BB(2);
Slope=BB(1); 
Pwave=P-Intercept-time*Slope;

Pstatic=P-Pwave;      %Hydrostatic pressure
Depth=Pstatic/Rho/g;
AverageDepth=mean(Depth);

%Optional figure to plot waterdepth as function of time
if (extraplot>2)
   figure
   plot(time,Depth);
   ylabel('depth (m)');
   xlabel('time (sec)');
   title(['average waterdepth ',heading]);

%Optional figure to plot wave pressure as function of time
   figure
   plot(time,Pwave);
   xlabel('time (sec)');
   ylabel('pressure (Pa)');
   title(['presure variations ',heading]);
end;
%============================================================================
%Time domain analysis
counter=0;
Pmin=0;
Pmax=0;
period=0;
start=1;
if (method==1)
%loop to find individual pressure waves, needed to find number of waves
%using mean period
  for k=1:n-1
    if(Pwave(k+1)*Pwave(k)<0 && Pwave(k+1)<0)  %new wave starts at k+1
        counter=counter+1;   % counter is number of found waves
        Pmax=0;
        Pmin=0;
    end;
    if(Pwave(k)>Pmax)  Pmax=Pwave(k);
    end;
    if (Pwave(k)<Pmin) Pmin=Pwave(k);
    end;
  end;
  %counter = number of waves found

  Tmean=n*interval/counter;
  %loop to estimate water level variations
  for k=1:n
        period=Tmean;
        L0=g/2/pi*period*period;
        if (Depth(k)/L0 < 0.36) 
            L=sqrt(g*Depth(k))*(1-Depth(k)/L0)*period;
        else
            L=L0;
        end;
        eta(k)=(Pwave(k))/Rho/g*cosh(2*pi/L*(Depth(k)+Bottom))/cosh(2*pi/L*Bottom);
  end;

  %loop to find individual waves from waterlevel record (eta)
  counter2=0;
  etamax=0;
  etamin=0;
  for k=1:n-1
    if(eta(k+1)*eta(k)<0 && eta(k+1)<0)  %new wave starts at k+1
        counter2=counter2+1;
        H(counter2)=etamax-etamin;
        etamax=0;
        etamin=0;
    end;
    if(eta(k)>etamax)  etamax=eta(k);
      end;
    if (eta(k)<etamin) etamin=eta(k);
      end;
  end;
  % vector H contains all individual waves
  % sort H to make exceedance graph
else 
  eta(1:n)=0;  
  Pmax=0;
  Pmin=0;
  % Method 2 starts here=================================================  
  %loop to find individual pressure waves, needed to find number of waves
  %using mean period
  for k=1:n-1
    if(Pwave(k+1)*Pwave(k)<0 && Pwave(k+1)<0)  %new wave starts at k+1
        counter=counter+1;   % counter is number of found waves
        TT(counter)=period;
        L0=g/2/pi*period*period;
        if (Depth(k)/L0 < 0.36) 
            L=sqrt(g*Depth(k))*(1-Depth(k)/L0)*period;
        else
            L=L0;
        end
        H(counter)=(Pmax-Pmin)/Rho/g*cosh(2*pi/L*(Depth(k)+Bottom))/cosh(2*pi/L*Bottom);
        Hunreduced=H(counter);
        Hreduced=H(counter);
        % when waveheight is too small H(counter) gets value NaN; this
        % cannot be plotted, therefore these cases get a value H(counter)=0
        if (isnan(H(counter))==1) H(counter)=0;end;
        if (H(counter)>steepness*L)
            H(counter)=steepness*L;
            Hreduced=H(counter);
            end;
        if (H(counter)<Hminimum)
             Hreduced=Hminimum;
             counter=counter-1;
            end;
        reduction=Hreduced/Hunreduced;    
        for jj=start:k   
            eta(jj)=(Pwave(jj))/Rho/g*cosh(2*pi/L*(Depth(jj)+Bottom))/cosh(2*pi/L*Bottom)*reduction;
            end;
        start=k+1;           % remember startpoint for next wave    
        Pmax=0;
        Pmin=0;
        period=0;
    end;
    period=period+interval;
    if(Pwave(k)>Pmax)  Pmax=Pwave(k);
    end;
    if (Pwave(k)<Pmin) Pmin=Pwave(k);
    end;
  end;
end;

  % counter = number of waves found
  % vector H contains all individual waves
  % sort H to make exceedance graph
  
if (extraplot>0)
 figure;
 subplot(4,1,1);
 plot(time,eta);
 xlabel('time (sec)');
 ylabel('waterlevel (m)');
 title([heading,date,tijd]);
end;
%end waterlevel loop

counter=length(H);
for k=1 : counter    
prob(k)= (counter-k+1)/counter;
end;

Hsorted=sort(H);
Hmean=mean(Hsorted);
Hrms = sqrt(mean(Hsorted.^2)); % rms wave height

from=round(2*counter/3);
to=round(counter);
Hs=mean(Hsorted(from:to)); %H 1/3, or significant wave height

Tsorted=sort(TT);
Tmean=mean(TT);
from=round(2*counter/3);
to=round(counter);
T3=mean(Tsorted(from:to)); %T 1/3, or significant wave peirodt

%calculation of the H2%
Twopercent=round(counter-counter/50);
if (Twopercent==counter)
  H2percentMeasured=NaN;
  else
  H2percentMeasured=Hsorted(Twopercent);
  end;
H2percRayleigh=1.4*Hs;
%caculation of the H1%
Onepercent=round(counter-counter/100);
if (Onepercent==counter)
  H1percentMeasured=NaN;
  else
  H1percentMeasured=Hsorted(Onepercent);
  end;
H1percRayleigh=1.5*Hs;
%calculation of the H0.1%
Onepermille=round(counter-counter/1000);
if (Onepermille==counter)
  H1permilleMeasured=NaN;
  else
  H1permilleMeasured=Hsorted(Onepermille);
  end;
H1permilleRayleigh=1.85*Hs;

% Compositie weibull ditribution for shallow water
% source: Wave height distributions on shallow foreshores 
%         Coastal Engineering, Volume 40, Issue 3, June 2000, Pages 161-182
%         Jurjen A Battjes, Heiko W Groenendijk

Htr=(0.35+5.8*TanAlfa)*AverageDepth; % Transistion wave height acc. to BG (eq. 8)

H3=Hrms*BG(Htr,3);
H10=Hrms*BG(Htr,10);
H2=Hrms*BG(Htr,0.02);
H1=Hrms*BG(Htr,0.01);
H01=Hrms*BG(Htr,0.001);

%plot Rayleigh graph
if(extraplot>0)
 h = axes('Position',[0 0 1 1],'Visible','off');
 axes('Position',[.45 .40 .45 .30])

  for i=1:counter
     probsorted2(i)=1-i/(counter+1);
  end;
 maxh=1.3*Hsorted(counter);
 step=1;
 if (maxh<5.1) step=0.5;end;
 if (maxh<2.6) step=0.25;end;
 if (maxh<1.0) step=0.1;end;
 if (maxh<0.5) step=0.05;end;
 if (maxh<0.1) step=0.01;end;
 if (maxh<0.01) step=0.001;end;
 xmark=[0:step:maxh];
 coef=Rayleigh(Hsorted,probsorted2,xmark,Hrms,Htr);
 title('Wave height distribution (m) ');
 string=num2str(Hmean,2);
 str(1)= {['H_{mean} (m)             ',string]};
 string=num2str(Hrms,2);
 str(2)= {['H_{rms} (m)              ',string]};
 string=num2str(Hs,2);
 str(3)= {['H_{1/3} (m)                 ',string]};
 string=num2str(Tmean,2);
 str(4)= {['T_{mean} (s)             ',string]};
 string=num2str(T3,2);
 str(5)= {['T_{1/3} (s)               ',string]};
 string=num2str(Htr,2);
 str(6)= {['H_{tr}(m)(acc BG)  '   ,string]};
 string=num2str(1/TanAlfa);
 str(7)= {['Bed slope 1:',string]};
 str(8)={'              obs    Rayleigh  BG(red)'};
 string=num2str(H2percentMeasured,3); stringB=num2str(H2percRayleigh,3);stringC=num2str(H2,3);
 str(9)= {['H_{2%}(m)   ',string,'    ',stringB,'      ',stringC]};
 string=num2str(H1percentMeasured,3); stringB=num2str(H1percRayleigh,3);stringC=num2str(H1,3);
 str(10)= {['H_{1%}(m)   ',string,'    ',stringB,'      ',stringC]};
 string=num2str(H1permilleMeasured,3); stringB=num2str(H1permilleRayleigh,3);stringC=num2str(H01,3);
 str(11)= {['H_{0.1%}(m) ',string,'    ',stringB,'      ',stringC]};
 set(gcf,'CurrentAxes',h)
 text(.1,.55,str,'FontSize',8)
end; 



%===================================================================
% simple script utilizing crosgk (by G. Klopman) to obtain
% spectral estimate
%===================================================================
% data  contains the data 
% N  is the number of samples per data segment (power of 2)
% M  is the number of frequency bins over which is smoothed (optional),
%    no smoothing for M=1 (default)
% DT is the time step (optional), default DT=1
% DW is the data window type (optional): DW = 1 for Hann window (default)
%                                        DW = 2 for rectangular window
% stats : display resolution, degrees of freedom (optimal, YES=1, NO=0)
%
% Output:
% P contains the (cross-)spectral estimates: column 1 = Pxx, 2 = Pyy, 3 = Pxy
% F contains the frequencies at which P is given load time series
             
DT = interval;
    data = Pwave;
    [P,F,dof]=crosgk(data,data,length(data),M,DT,1,0);

 %recalcultate pressure spectrum to energy spectrum
  energy=1:length(F);  % length (F) is number of frequency bins
  for i=1:length(F)
      energy(i)=0;
  end;
  m0=0;   %zero-th moment
  m1=0;   %first moment
  m2=0;   %second moment
  m01=0;  %first negative moment {m(-1,0)}
  deltaF= F(31)-F(30);

  if iEnd>length(F)
     iEnd=int32(length(F));
  end;
  emax=0;
  prmax=0;  %maximum value of pressure energy
  % claculation loop to transform pressure spectrum to energy spectrum and
  % to calculate the moments of the spectrum
  % low and high frequencies are deleted, range from 200 to 0.2*max
  % frequency bin, 200 means f= 200*deltaF, which is approx. 30 seconds
  % 0.2*length(F)*deltaF = 0.4, so Tmin - 2.5 seconds
  
  %loop to determine maximum value of pressure spectrum
 
  %find maximum pressure value in spectrum
  for i=iBegin:iEnd
      if (sqrt(P(i,1))>prmax) prmax=sqrt(P(i,1));
      end;
  end;
  
  for i=iBegin:iEnd
      T=1/F(i);
      pr=sqrt(P(i,1));   %pr=pressure value in pressure spectrum
      L0=1.56*T*T;
      if (AverageDepth/L0<0.36) 
          L=sqrt(g*AverageDepth)*(1-AverageDepth/L0)*T;
          else
          L=L0;
          end;
      energy(i)=(pr/(Rho*g)*cosh(2*pi/L*(AverageDepth+Bottom))/cosh(2*pi/L*Bottom))^2; % e=energydensity in Hz/m2
      if (pr<cutoff*prmax)
          energy(i)=0;
          end;
      if energy(i)>emax 
          emax=energy(i);
          Tpeak=T;
          end;
      m0 =m0 +energy(i)*deltaF;
      m1 =m1 +energy(i)*deltaF*F(i);
      m2 =m2 +energy(i)*deltaF*F(i)^2;
      if (F(i)>0)
          m01=m01+energy(i)*deltaF/F(i);
          end;
  end;
  
  Hm0=4*sqrt(m0);    %one may assume Hs = Hm0
  Hrmss=sqrt(8*m0);   %root mean square height from spectrum
  Tm=sqrt(m0/m2);    %spectral approximation of mean period
  T01=m0/m1;         %Period based on first moment 
  T10=m01/m0;        %Period based on first negative moment

  
  % plot spectrum and print output
  if(extraplot>0)
   h = axes('Position',[0 0 1 1],'Visible','off');
   axes('Position',[.45 .05 .45 .20])
   plot(F,energy)
   axis([low high 0 10000]) 
   axis 'auto y'
   xlabel('frequency [Hz]');
   ylabel ('Energy m^2/Hz');
   title('energy spectrum ');
   string=num2str(Hrmss,2);
   str(1)= {['H_{rms} (m) ',string]};
   string=num2str(Hm0,2);
   str(2)= {['H_{m0} (m)  ',string]};
   string=num2str(Tm,2);
   str(3)= {['T_{m} (s)    ',string]};
   string=num2str(T01,2);
   str(4)= {['T_{m0,1} (s) ',string]};
   string=num2str(T10,2);
   str(5)= {['T_{m-1,0}(s) ',string]};
   string=num2str(Tpeak,2);
   str(6)= {['T_{peak}(s)  ',string]};
   str(7)={' '};
   string=num2str(Bottom,2);
   str(8)={['sensor above bed (m)    ',string]};
   string=num2str(AverageDepth+Bottom,2);
   str(9)={['Average depth (m)        ',string]};
   string=num2str(Rho);
   str(10)={['water density (kgm^{3})    ',string]};
   str(11)={' '};
   set(gcf,'CurrentAxes',h)
   text(.1,.15,str,'FontSize',8)
  end;
              
 %Optional plot pressure spectrum
if (extraplot>1)
    figure
    plot(F,P(:,1))     % P is pressure^2/Hz
    axis ([low high 0 1500])
    axis 'auto y'
    xlabel('frequency [Hz]');
    ylabel ('pressure Pa^2/Hz');
    title(['pressure spectrum ',heading]);
end;

StUitvoer(ii,1)=date;
StUitvoer(ii,2)=tijd;

uitvoer(ii,1)=ii;
uitvoer(ii,2)=AverageDepth+Bottom;
uitvoer(ii,4)=Hs;
uitvoer(ii,5)=Hm0;
uitvoer(ii,6)=Tmean;
uitvoer(ii,7)=Tm;
uitvoer(ii,8)=T10;
uitvoer(ii,9)=Tpeak;
disp(ii);
end;

MeanDepth=0;
for (ii=StartFile:NofFiles)
MeanDepth=MeanDepth+uitvoer(ii,2);
end;
MeanDepth=MeanDepth/(NofFiles-StartFile);
for(ii=StartFile:NofFiles)
uitvoer(ii,3)=uitvoer(ii,2)-MeanDepth;
end;
savefile=([heading,'.mat']);
save (savefile,'StUitvoer','uitvoer');

