%define basic variables
clear;
clc;
Rho=1000;
g=9.81;
Avolt=254;
Bvolt=-43250;
load rawdata.txt;
%transformation from Volts to pressure using calibration constants Avolt
%and Bvolt
%Avolt has dimension Pa/V, Bvolt has dimension Pa

P=Avolt*rawdata+Bvolt;  
%interval is the sample frequency interval of the sensor
interval=.25;
n=numel(rawdata); %count number of samples
tottime=n*interval; %calculate total duration of observation in sec
ttime=(interval:interval:tottime); %create array with time
time=ttime.'; %change orientation of matrix
% calculate regression coefficients to compensate for change in waterlevel
% during the observations
%plot(time,P);
B = [ones(length(time),1) time] \ P;
Slope=B(2);
Intercept=B(1);
Pwave=P-Intercept'-time*Slope;

%figure
%plot(time,Pwave);
%xlabel('time (sec)');
%ylabel('pressure (Pa)');

Pstatic=P-Pwave;
Depth=Pstatic/Rho/g;
AverageDepth=mean(Depth);

%figure
%plot(time,Depth);
%ylabel('depth (m)');
%xlabel('time (sec)');

counter=0;
Pmin=0;
Pmax=0;
period=0;
upgrade=1.00; %calabration coeffient for transformation pressure to height
for k=1:n-1
    if(Pwave(k+1)*Pwave(k)<0 && Pwave(k+1)<0)  %new wave starts at k+1
        counter=counter+1;
        T(counter)=period;
        L0=g/2/pi*period*period;
        if (Depth(k)/L0 < 0.36) 
            L=sqrt(g*Depth(k))*(1-Depth(k)/L0)*period;
        else
            L=L0;
        end;
        H(counter)=(Pmax-Pmin)/Rho/g*cosh(2*pi/L*Depth(k));
        % H(i) is wave heigth calculated from individual pressure variation
        Pmax=0;
        Pmin=0;
        period=0;
    end;
    if(Pwave(k)>Pmax)  Pmax=Pwave(k);
    end;
    if (Pwave(k)<Pmin) Pmin=Pwave(k);
    end;
    period=period+interval;
end;
% counter = number of waves found
Hsorted=sort(H);  % sorted waves in order of height
%statistic of Hsorted is not used in this program 

Tsorted=sort(T);
Tmean = mean (Tsorted);

%loop to estimate water level variations
for k=1:n
        period=Tmean;
        L0=g/2/pi*period*period;
        if (Depth(k)/L0 < 0.36) 
            L=sqrt(g*Depth(k))*(1-Depth(k)/L0)*period;
        else
            L=L0;
        end;
        eta(k)=upgrade*(Pwave(k))/Rho/g*cosh(2*pi/L*Depth(k));
end;

%eta is the calculated waterlevel as function of time
figure
plot(time,eta);
xlabel('time (sec)');
ylabel('waterlevel (m)');
%end waterlevel loop

%start loop to determine individual waves from waterlevel
etamin=0;
etamax=0;
counter=0;
for k=1:n-1
    if(eta(k+1)*eta(k)<0 && eta(k+1)<0)  %new wave starts at k+1
        counter=counter+1;
        Hxx(counter)=etamax-etamin; %Hxx is wave height using recreated surface
        etamin=0;
        etamax=0;
    end;
    if(eta(k)>etamax)  etamax=eta(k);
    end;
    if (eta(k)<etamin) etamin=eta(k);
    end;
end;
%end loop to determine individual waves

Hsorted3=sort(Hxx);   % sorted waves in order of height

%optional loop to ignore wave less then Hmin (m)
Hmin =0.05;
kk=0;
for k=1:counter
    if  Hsorted3(k)>Hmin
        kk=kk+1;
        Hsorted2(kk)=Hsorted3(k);
    end;
end;
counter=kk;  %reduce number of waves

for k=1 : counter    
prob(k)= (counter-k+1)/counter;
end;

Hmean=mean(Hsorted2);
Hrms = sqrt(mean(Hsorted2.^2)) % rms wave height

%calculation of the H1/3
from=round(2*counter/3);
to=round(counter);
Hs=mean(Hsorted2(from:to))   % significant wave height

%calculation of the H2%
Twopercent=round(counter-counter/50);
if (Twopercent==counter)
    H2percenteMeasured=NaN;
else
    H2percentMeasured=Hsorted2(Twopercent);
end;
H2percentMeasured
H2percRayleigh=1.4*Hs

%caculation of the H1%
Onepercent=round(counter-counter/100);
if (Onepercent==counter)
    H1percenteMeasured=NaN;
else
    H1percentMeasured=Hsorted2(Onepercent);
end;
H1percentMeasured
H1percRayleigh=1.5*Hs

%calculation of the H0.1%
Onepermille=round(counter-counter/1000);
if (Onepermille==counter)
    H1permilleMeasured=NaN;
else
    H1permilleMeasured=Hsorted2(Onepermille);
end;
H1permilleMeasured
H1permilleRayleigh=1.85*Hs

TanAlfa = 0.001;   %bedslope
Htr=(0.35+5.8*TanAlfa)*AverageDepth % Transistion wave height acc. to BG
HtrNorm=Htr/Hrms   %normalised transition height

figure
probplot('rayleigh',Hsorted2(1:counter));
hold all;
plot(Hs,2,'--rs','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',5);
plot(Hrms,1,'--rs','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',5) ; 
plot(H2percRayleigh,2.8,'--rs','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',5);            
plot(H1percRayleigh,3.03,'--rs','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',5);
plot(H1permilleRayleigh,3.7,'--rs','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',5) ;           
grid on;
xlabel('wave height (m)');

